#!/bin/sh

BASE_DIR=$(dirname "$0")

JAR_CNT=$(find $BASE_DIR/*.jar | wc -l)

if [ $JAR_CNT -eq 0 ]; then
  echo "JAR not found in $BASE_DIR"
  exit 1
fi

if [ $JAR_CNT -gt 1 ]; then
  echo "Multiple JARs found in $BASE_DIR"
  exit 1
fi

JAR_NAME=$(find $BASE_DIR/*.jar)

if [ -z "$JAR_NAME" ]; then
  echo "JAR_NAME not found in $BASE_DIR"
  exit 1
fi

echo ">>> JAR_NAME = $JAR_NAME <<<"

API_NAME=$(basename $JAR_NAME .jar)
echo ">>> API_NAME = $API_NAME <<<"

#PID -> PID File
PID_FILE=$BASE_DIR/$API_NAME\.pid

#Usage, hint for input parameters
function usage() {
    echo ">>>Usage: sh app-run.sh [start [JAVA_OPTS]|stop|configure]<<<"
    exit 1
}

#Check the application is running or not
function is_running() {
  pid=`ps -ef | grep $JAR_NAME | grep -v grep | awk '{print $2}' `
  if [[ ! -z $pid ]]; then
    return 1
  fi
  return 0;
}

#Stop the application
function stop() {
  if [ -e $PID_FILE ]; then
    PID=$(cat $PID_FILE)
    echo ">>>Start kill PID = $PID<<<"
    kill $PID
    rm -f $PID_FILE
  fi

  sleep 5
  
  pid=`ps -ef | grep $JAR_NAME | grep -v grep | awk '{print $2}' `
  if [[ ! -z $pid ]]; then
    echo "Start force kill >>> PID = $pid <<<"
    kill -9 $pid
  fi
}

#Start the application
function start() { 
  export JAVA_OPTS

  for arg in "$@"
  do
    if [[ $arg == -D* ]]; then
      JAVA_OPTS="$JAVA_OPTS $arg"
    fi
  done

  stty -echo
  printf "Master Keystore Password: "
  read PASSWORD
  stty echo
  printf "\n"

  env "crp.system.keystore.master.password=$PASSWORD" sh -c """
    nohup java -jar $JAVA_OPTS $JAR_NAME > /dev/null 2>&1 & 
    echo \$! > $PID_FILE
    exit 0
  """
  PID=$(cat $PID_FILE)
  echo ">>> start $JAR_NAME successed PID=$PID <<<"
}

function configure() {
  java -jar $JAR_NAME configure
}

case "$1" in
  "start")
    start ${@:2}
    ;;
  "stop")
    stop
    ;;
  "status")
    status
    ;;
  "configure")
    configure
    ;;
  *)
    usage
    ;;
esac
exit 0
